require 'dbc'
require 'csv'
require 'sqlescape'
esc = sqlescape.EscapeFunction(db.SQLITE)

-- create this directory and use for all files
-- also use this as the  directory for the text ODBC connection
WORKING_DIR = [[C:\Temp\ErrorMsg\]]

-- put SQLite in the working directory
SQLITE_DB = WORKING_DIR..'Errors.sqlite'

DB_sqlite = dbc.Connection{
   api = db.SQLITE,
   name = SQLITE_DB
}

local function trace(a,b,c,d) return end

function main() 
   CreateErrorTable()
    
   -- insert errors into database
   insertErrors()
   
   -- check what was inserted
   DB_sqlite:query('SELECT * FROM Error')
      
   -- update Status, Comment, Action etc fields
   updateErrors()
         
   -- export ALL to a CSV file for excel
   exportCSV_All()
   
   -- export rows  marked "show" to a CSV file for excel
   exportCSV_Show()

   -- export rows  marked "show" to a CSV file for excel
   exportCSV_ShowNoLua()

   -- export rows  marked "show" to a CSV file for excel
   exportCSV_ShowLua()

   -- export rows without comment to a CSV file for excel
   exportCSV_NoComment()
      
   -- example exporting only Message and Action
   --exportCSV_MessageAction()
end

-- create Error table - the SQL used is ***SQLite*specific***
function CreateErrorTable()
   local Sql=
   'CREATE TABLE IF NOT EXISTS Error\n'..
   '(\n'..
     'Message TEXT NOT NULL PRIMARY KEY,\n'..
     'Type TEXT NULL,\n'..
     'Status TEXT NULL,\n'..
     'Source TEXT NULL,\n'..
     'LogSource TEXT NULL,\n'..
     'Location TEXT NULL,\n'..
     'Description TEXT NULL,\n'..
     'Comment TEXT NULL,\n'..
     'Action TEXT NULL\n'..
   ');' 
   trace(Sql)
   DB_sqlite:execute(Sql,true) 
end

function isFile(fname)
   local f = io.open(fname,'rb')
   if f then
      io.close(f)
      return true
   else
      return false
   end
end

function backupSqlite()
   trace(os.date():sub(10,17):gsub(":","-"))
   local to = WORKING_DIR..'ErrorsBackup_'..
      os.date():sub(10,17):gsub(":","-")..'.sqlite'
   os.execute('copy '..SQLITE_DB..' '..to)
end

function insertErrors()
   if isFile(WORKING_DIR..'ErrorsIn.csv') then
      backupSqlite()
      local CSV = csv.load(WORKING_DIR..'ErrorsIn.csv')
      -- find Id for Message, LogSource (Source Name)
      local IdM = 0
      local IdL = 0
      for i=1,#CSV[1] do
         if CSV[1][i] == 'Message' then IdM = i end
         if CSV[1][i] == 'Source Name' then IdL = i end
      end
      -- insert into SQLite
      local sql = ''
      local src = '***'
      for i=2,#CSV do
         sql = [[SELECT COUNT(Message) FROM Error 
         WHERE Message = ']]..esc(CSV[i][IdM])..[[']]
         local cnt = DB_sqlite:query(sql)[1]["COUNT(Message)"]:nodeValue()
         if IdL ~= 0 then 
            src = CSV[i][IdL] 
         end
         sql = [[INSERT INTO Error(Message,Type,Status,Source,
            LogSource,Location,Description,Comment,Action) 
            VALUES(']]..esc(CSV[i][IdM])..[[','','','',']]..
            src..[[','','','','')]]
         trace(sql)
         if cnt == '0' then
            DB_sqlite:execute(sql,true) 
         end
      end
   end
end

function updateErrors()
   if isFile(WORKING_DIR..'ErrorsDoc.csv') then
      backupSqlite()
      CSV = csv.load(WORKING_DIR..'ErrorsDoc.csv')
      trace(CSV,#CSV)
      -- find Id for columns to update
      local msgId = 0 
      local typId = 0
      local staId = 0
      local srcId = 0
      local locId = 0
      local dscId = 0
      local cmtId = 0
      local actId = 0
      for i=1,#CSV[1] do
         if CSV[1][i] == 'Message' then msgId = i end
         if CSV[1][i] == 'Type' then typId = i end
         if CSV[1][i] == 'Status' then staId = i end
         if CSV[1][i] == 'Source' then srcId = i end
         if CSV[1][i] == 'Location' then locId = i end
         if CSV[1][i] == 'Description' then dscId = i end
         if CSV[1][i] == 'Comment' then cmtId = i end
         if CSV[1][i] == 'Action' then actId = i end
      end
      local msg = ''
      local typ = ''
      local sta = ''
      local src = ''
      local loc = ''
      local dsc = ''
      local cmt = ''
      local act = ''
      trace(#CSV)
      for i=2,#CSV do
         msg = esc(CSV[i][msgId])
         if CSV[i][typId] == nil then 
            typ = 'NULL' 
         else
            typ = esc(CSV[i][typId]) 
         end        
         if CSV[i][staId] == nil then 
            sta = 'NULL' 
         else
            sta = esc(CSV[i][staId]) 
         end
         if CSV[i][srcId] == nil then 
            src = 'NULL' 
         else
            src = esc(CSV[i][srcId]) 
         end       
         if CSV[i][locId] == nil then 
            loc = 'NULL' 
         else 
            loc = esc(CSV[i][locId]) 
         end        
         if CSV[i][dscId] == nil then 
            dsc = 'NULL' 
         else
            dsc = esc(CSV[i][dscId]) 
         end         
         if CSV[i][cmtId] == nil then 
            cmt = 'NULL' 
         else
            cmt = esc(CSV[i][cmtId]) 
         end        
         if CSV[i][actId] == nil then 
            act = 'NULL' 
         else
            act = esc(CSV[i][actId]) 
         end        
         sql = [[UPDATE Error SET Type = ']]..typ..[[',
         Status =']]..sta..[[',
         Source =']]..src..[[',
         Location =']]..loc..[[',
         Description =']]..dsc..[[',
         Comment =']]..cmt..[[',
         Action =']]..act..[['
         WHERE Message = ']]..msg..[[']]
         trace(sql)
         DB_sqlite:execute(sql,true) 
      end
   end
end

function exportCSV(sql,csvheader,file)
   local result = ''
   local CSV = csvheader
   result = DB_sqlite:query(sql)
   for i = 1, #result do
      if i > 1 then CSV = CSV..'\n' end
      for j = 1, #result[i] do
         if j > 1 then CSV = CSV..',' end
         CSV = CSV..csv.escape(result[i][j])
      end
   end
   if isFile(WORKING_DIR..file) == false then
      os.execute('type NUL >'..WORKING_DIR..file)
   end      
   f = io.open(WORKING_DIR..file,'w')
   f:write(CSV)
   f:close()
end

function exportCSV_All()
   local sql = [[SELECT * FROM Error ORDER BY Message]]
   local csvheader = 'Message,Type,Status,'..
      'Source,LogSource,Location,'..
      'Description,Comment,Action\n'
   local file = 'ErrorsOutAll.csv'
   exportCSV(sql,csvheader,file)
end

function exportCSV_Show()
   local sql = [[SELECT * FROM Error 
      WHERE (Status = 'show')
      ORDER BY Message]]
   local csvheader = 'Message,Type,Status,'..
      'Source,LogSource,Location,'..
      'Description,Comment,Action\n'
   local file = 'ErrorsOutShow.csv'
   exportCSV(sql,csvheader,file)
end

function exportCSV_ShowNoLua()
   local sql = [[SELECT * FROM Error 
      WHERE (Status = 'show' AND Type <> 'Lua debug')
      ORDER BY Message]]
   local csvheader = 'Message,Type,Status,'..
      'Source,LogSource,Location,'..
      'Description,Comment,Action\n'
   local file = 'ErrorsOutShowNoLua.csv'
   exportCSV(sql,csvheader,file)
end

function exportCSV_ShowLua()
   local sql = [[SELECT * FROM Error 
      WHERE (Status = 'show' AND Type = 'Lua debug')
      ORDER BY Message]]
   local csvheader = 'Message,Type,Status,'..
      'Source,LogSource,Location,'..
      'Description,Comment,Action\n'
   local file = 'ErrorsOutShowLua.csv'
   exportCSV(sql,csvheader,file)
end

function exportCSV_NoComment()
   -- select newly imported data = no comment
   -- use when adding fields and you want to add
   -- Comments and Actions
   local sql = [[SELECT * FROM Error 
      WHERE (Comment = '' OR Comment IS NULL)
      ORDER BY Message]]
   local csvheader = 'Message,Type,Status,'..
      'Source,LogSource,Location,'..
      'Description,Comment,Action\n'
   local file = 'ErrorsOutNoComment.csv'
   exportCSV(sql,csvheader,file)
end

function exportCSV_MessageAction()
   -- example of how easy it is to do a different export
   -- export Message and Action only for rows  marked "show" to a CSV
   local sql = [[SELECT Message, Action FROM Error 
      WHERE (Status = 'show')
      ORDER BY Message]]
   local csvheader = 'Message,Action\n'
   local file = 'ErrorsOutShowAction.csv'
   exportCSV(sql,csvheader,file)
end
