-- $Revision: 1.7 $
-- $Date: 2012-05-01 14:02:23 $

-- Module which can be used to get a SQL escape function for a given database 
-- Example usage: 
-- local E = sqlescape.EscapeFunction(db.MY_SQL)
-- local V = E("This data contains quote ' characters") 

sqlescape = {} 

local function BackSlashEscape(Value) 
   Value = Value:gsub("\\", "\\\\")
   Value = Value:gsub('"', '\\"') 
   Value = Value:gsub("'", "\\'") 
   return Value 
end 

local function DoubleQuoteEscape(Value) 
   Value = Value:gsub('"', '""') 
   Value = Value:gsub("'", "''") 
   return Value 
end 
 
local function SingleQuoteEscape(Value) 
   Value = Value:gsub("'", "''") 
   return Value 
end 

local function OracleEscape(Value) 
   Value = Value:gsub("\\", "\\\\")
   Value = Value:gsub("'", "''") 
   return Value 
end 
 
local Lookup ={ 
   [db.MY_SQL]      = BackSlashEscape, 
   [db.ORACLE_OCI]  = OracleEscape, 
   [db.ORACLE_ODBC] = OracleEscape, 
   [db.SQLITE]      = SingleQuoteEscape, 
   [db.SQL_SERVER]  = DoubleQuoteEscape, 
   [db.POSTGRES]    = BackSlashEscape, 
   [db.DB2]         = DoubleQuoteEscape, 
   [db.INFORMIX]    = BackSlashEscape, 
   [db.INTERBASE]   = DoubleQuoteEscape, 
   [db.FILEMAKER]   = BackSlashEscape, 
   [db.SYBASE_ASA]  = DoubleQuoteEscape, 
   [db.SYBASE_ASE]  = DoubleQuoteEscape 
} 
 
function sqlescape.EscapeFunction(DatabaseType) 
   return Lookup[DatabaseType] 
end